window.Gleam.initSearch([{"doc":"bravo","title":"bravo","content":"# bravo\n### [v3.0.0-rc.1 (This is a breaking update!)](https://github.com/Michael-Mark-Edu/bravo/blob/main/CHANGELOG.md#v300)\n\n[![Package Version](https://img.shields.io/hexpm/v/bravo)](https://hex.pm/packages/bravo)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/bravo/)\n\nComprehensive ETS bindings for Gleam.\n\nThis library is only compatible with the Erlang target!\n\n## What is an ETS?\n\nETS stands for Erlang Term Storage, and it is a data structure that stores collections of data\n(objects) that are addressable by keys.\n\n### Why should I use ETS over, say, a `Dict`?\n\nETS tables have the following properties that make them distinct from `Dict`s:\n- ETS tables are inherently mutable, allowing for faster operations especially when working with\n  large amounts of data.\n- ETS tables are not garbage collected, allowing them to be accessed from anywhere in a process\n  without needing to pass return values around. This does come at the cost of forcing the user\n  to manually delete the table once they are done with it.\n- ETS tables offer a wide array of customization options that `Dict`s do not, such as the options\n  to be ordered, allow multiple objects per key, which element of the object acts as the key,\n  and controlling read/write access from external processes.\n- ETS tables offer increased functionality compared to `Dict`s, natively allowing functions such as\n  complex searching, reading/writing the ETS to a file, and having much more complex entries.\n- ETS tables work well with concurrency.\n\n### Why should I use Bravo over other libraries such as Carpenter?\n\nA difficulty of implementing ETS tables in Gleam is that Gleam is a statically-typed language and\nETS tables are very much designed for the dynamically-typed Erlang. Carpenter ensures type safety\nby restricting objects to simple key-value pairs, much like a `Dict`.\n\nBravo, however, does not impose much of any restriction at all. The only restriction that is in\nplace is that all objects in a table must have the same type, but that type can be literally\nanything. In an earlier version of this package, even this wasn&#39;t a restriction, but that resulted\nin unsafe and un-Gleamlike code.\n\nA goal of Bravo is to be fully comprehensive, meaning that I intend on implementing every single\nETS function into the library. This goal may very well be infeasible, but I will pursue it anyway.\n\n## Installation\nBravo depends on the `gleam_erlang` package, but otherwise does not have any other dependency.\n\n```sh\ngleam add bravo gleam_erlang\n```\n\n## Example\n```gleam\nimport bravo\nimport bravo/uset\nimport gleam/io\nimport gleam/option.{Some}\n\npub fn main() {\n  // Create a new ETS table. There are multiple options, but here we are using\n  // a USet (an alias for &quot;set&quot; in Erlang terms)\n  let assert Ok(table) = uset.new(&quot;MyTable&quot;, 1, bravo.Public)\n\n  // We can then insert tuples into this table\n  uset.insert(table, [#(&quot;Hello&quot;, &quot;world!&quot;)])\n\n  // Then we can lookup the object from the table\n  let assert Some(object) = uset.lookup(table, &quot;Hello&quot;)\n  io.print(object.0 &lt;&gt; &quot;, &quot; &lt;&gt; object.1) // &quot;Hello, world!&quot;\n\n  // ETS tables have static lifetimes,\n  // so don&#39;t forget to delete them when you&#39;re done!\n  uset.delete(table)\n}\n```\n\n## Bravo in Five Minutes\n\nFirst, you must choose a table type. Bravo offers four options:\n\n- `USet` (set): Keys may only occur once per table, and objects are unordered.\n- `OSet` (ordered set): Keys may only occur once per table, and objects are ordered (this comes at a performance cost).\n- `Bag` (bag): Keys may occur multiple times per table, but objects cannot be copied verbatim.\n- `DBag` (duplicate bag): Keys may occur multiple times per table, and verbatim copies of an object can be stored.\n\nFor the sake of demonstration, this section will use the `USet`.\n\nFirst, we must create a `USet`. We must pass in a name, the table&#39;s `keypos` (recommended to leave\nthis as 1) and a access specifier (not relevant unless you&#39;re working with processes/actors).\n```gleam\nimport bravo\nimport bravo/uset\n\nlet assert Ok(table) = uset.new(&quot;MyTable&quot;, 1, bravo.Public)\n```\nThis will return a `Result` containing the table, if the table creation succeeded. If there\nalready exists a table with the same name, this function will fail.\n\nThis table will be empty, so we&#39;ll need to insert some _objects_ into it. An object can be of any\ntype, but it should ideally be a tuple. (Note: it is better to use a non-tuple over a size-1 tuple!\nIt is theoretically more memory efficient to use non-tuples, and some functions like `file2tab`\ngenerate tables with the non-tuple variant regardless of what you put in.)\n\nTo insert, we use the `insert` function, which takes in a list of objects.\n```gleam\nuset.insert(table, [#(&quot;Hello&quot;, &quot;World&quot;)])\n```\n\n_A table can only accept new objects that are the same type!_ This is enforced at compile time,\nand the type of the table is determined via type inference through the first `insert` call. (Tip:\nif you need a table with multiple object types, it is valid to use `dynamic.from` in the `insert`\ncall to do this, but it introduces unsafety into your code that must be resolved using decoders.)\n\nNow that we have an object in the table, we can look it up using the `lookup` function. It\nsimply takes in the table and a key (in this case, the first element of the tuple), and returns the\nobject (or list of objects in the case of `Bag` and `DBag`) if it was found.\n```gleam\nuset.lookup(table, &quot;Hello&quot;)\n|&gt; should.equal(Some(#(&quot;Hello&quot;, &quot;World&quot;)))\n```\n\nTheoretically, this is all you need to be able to use Bravo, but there are more functions that\nmay be useful:\n- `delete`: Completely deletes a table\n- `delete_key`: Deletes all objects with the key\n- `insert_new`: Inserts an object(s), but only if there does not exist any object with the same key\n- `take`: Looks up an object, returns it, then removes the object from the table\n- `member`: Gets if a table contains at least one object with the given key\n- `tab2file`: Saves the ETS table to a file.\n- `file2tab`: Reverses the above function. Due to type safety, this needs a dynamic decoder.\n\nETS tables are powerful tools provided natively by Erlang and the Beam VM, so make use of them!\n\nFurther documentation can be found at &lt;https://hexdocs.pm/bravo&gt;.\n","url":"index.html"},{"doc":"bravo","title":"Access","content":"pub type Access {\n  Public\n  Protected\n  Private\n}\n Access specifiers for ETS tables. Affects how other processes can interact with the table.\n\nPublic\n Any process can read or write to the `USet`.\n\nProtected\n Any process can read the `USet`. Only the owner process can write to it.\n\nPrivate\n Only the parent process can read or write to the `USet`.\n\nSynonyms:\nbravo.Access\nbravo Access","url":"bravo.html#Access"},{"doc":"bravo","title":"BravoError","content":"pub type BravoError {\n  NonPositiveKeypos\n  DecodeFailure\n  ErlangError(String)\n}\n The error type all Bravo functions use.\n\nNonPositiveKeypos\n Thrown if `new` is passed a `keypos` &lt;= 0\n\nDecodeFailure\n Thrown if `file2tab` isn&#39;t able to successfully decode all elements in a table file.\n\nErlangError(String)\n Runtime error caught in FFI. A non-exhaustive list of possible causes:\n - `new` tried to create a table with the same name as another.\n - `tab2file` couldn&#39;t create a file successfully.\n - `file2tab` tries to read non-ETS or corrupted data.\n\nSynonyms:\nbravo.BravoError\nbravo BravoError","url":"bravo.html#BravoError"},{"doc":"bravo","title":"bravo","content":" Contains types used by all modules in the Bravo package.","url":"bravo.html"},{"doc":"bravo/bag","title":"Bag","content":"pub opaque type Bag(t)\n A bag table. Keys may occur multiple times per table, but objects cannot be copied verbatim.\n\n\nSynonyms:\nbravo/bag.Bag\nbravo/bag Bag","url":"bravo/bag.html#Bag"},{"doc":"bravo/bag","title":"delete","content":"pub fn delete(bag: Bag(a)) -&gt; Bool\n Deletes a `Bag`.\n\n Table lifetime is static, and memory is only freed when the owner process is killed! Don&#39;t forget to call this function!\n\nSynonyms:\nbravo/bag.delete\nbravo/bag delete","url":"bravo/bag.html#delete"},{"doc":"bravo/bag","title":"delete_all_objects","content":"pub fn delete_all_objects(bag: Bag(a)) -&gt; Nil\n Deletes all objects in the `Bag`. This is atomic and isolated.\n\nSynonyms:\nbravo/bag.delete_all_objects\nbravo/bag delete_all_objects","url":"bravo/bag.html#delete_all_objects"},{"doc":"bravo/bag","title":"delete_key","content":"pub fn delete_key(bag: Bag(a), key: b) -&gt; Nil\n Deletes all objects addressed by `key`, if any exist. If nothing does, this does nothing.\n\nSynonyms:\nbravo/bag.delete_key\nbravo/bag delete_key","url":"bravo/bag.html#delete_key"},{"doc":"bravo/bag","title":"delete_object","content":"pub fn delete_object(bag: Bag(a), object: a) -&gt; Nil\n Deletes a specific object in the `Bag`. Other objects with the same key are unaffected.\n\nSynonyms:\nbravo/bag.delete_object\nbravo/bag delete_object","url":"bravo/bag.html#delete_object"},{"doc":"bravo/bag","title":"file2tab","content":"pub fn file2tab(\n  filename: String,\n  verify: Bool,\n  decoder: fn(Dynamic) -&gt; Result(a, b),\n) -&gt; Result(Bag(a), BravoError)\n Creates a `Bag` from file `filename` that was previously created by `tab2file`.\n\n For type safety reasons, a dynamic decoder must be provided, and the decoder must not fail for all objects in the table.\n\n If the flag `verify` is set, then checks are performed to ensure the data is correct. This can be slow if `tab2file` was ran with `md5sum` enabled.\n\n Size-1 tuples are handled uniquely internally and are treated as non-tuples when saved to disk, so you must decode the non-tuple version of the type.\n\n Can have error types `DecodeFailure` and `ErlangError`.\n\nSynonyms:\nbravo/bag.file2tab\nbravo/bag file2tab","url":"bravo/bag.html#file2tab"},{"doc":"bravo/bag","title":"insert","content":"pub fn insert(bag: Bag(a), objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `Bag`.\n\n Returns a `Bool` representing if the inserting succeeded.\n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the `keypos` of the `Bag` is greater than the object tuple size or if the input list is empty.\n\n If an object with the same key already exists, then the old object will be overwritten with the new one.\n\nSynonyms:\nbravo/bag.insert\nbravo/bag insert","url":"bravo/bag.html#insert"},{"doc":"bravo/bag","title":"insert_new","content":"pub fn insert_new(bag: Bag(a), objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `Bag`. Unlike `insert`, this cannot overwrite objects and will return false if it tries to do so.\n\n Returns a `Bool` representing if the inserting succeeded.\n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the `keypos` of the `Bag` is greater than the object tuple size or if the input list is empty.\n\nSynonyms:\nbravo/bag.insert_new\nbravo/bag insert_new","url":"bravo/bag.html#insert_new"},{"doc":"bravo/bag","title":"lookup","content":"pub fn lookup(bag: Bag(a), key: b) -&gt; List(a)\n Gets a list of objects from a `Bag`.\n\n Returns an list containing the objects, if any match.\n\nSynonyms:\nbravo/bag.lookup\nbravo/bag lookup","url":"bravo/bag.html#lookup"},{"doc":"bravo/bag","title":"member","content":"pub fn member(bag: Bag(a), key: b) -&gt; Bool\n Returns whether a `Bag` contains an object at `key`.\n\nSynonyms:\nbravo/bag.member\nbravo/bag member","url":"bravo/bag.html#member"},{"doc":"bravo/bag","title":"new","content":"pub fn new(\n  name: String,\n  keypos: Int,\n  access: Access,\n) -&gt; Result(Bag(a), BravoError)\n Creates a new ETS table configured as a bag: keys may only occur multiple times per table, but objects cannot be copied verbatim.\n\n `name`: An atom representing the name of the `Bag`. There may only be one ETS table associated with an atom.\n `keypos`: The index (1-indexed) that represents the key position of the object. This function fails if this is less than 1.\n `access`: Determines how visible the table is to other processes.\n - `Public`: Any process can read or write to the `Bag`.\n - `Protected`: Any process can read the `Bag`. Only the owner process can write to it.\n - `Private`: Only the parent process can read or write to the `Bag`.\n\n Returns a result of the created `Bag`, which can be used by other functions in this module.\n Can have error types `ErlangError` and `NonPositiveKeypos`.\n\nSynonyms:\nbravo/bag.new\nbravo/bag new","url":"bravo/bag.html#new"},{"doc":"bravo/bag","title":"tab2file","content":"pub fn tab2file(\n  bag: Bag(a),\n  filename: String,\n  object_count: Bool,\n  md5sum: Bool,\n  sync: Bool,\n) -&gt; Result(Nil, BravoError)\n Saves a `Bag` as file `filename` that can later be read back into memory using `file2tab`.\n\n There are three configuration flags with this function:\n - `object_count`: Stores the number of objects in the table in the footer. This can detect truncation.\n - `md5sum`: Stores a md5 checksum of the table and its objects. This can detect even single bitflips, but is computationally expensive.\n - `sync`: Blocks the process until the file has been successfully written.\n\n Can have error type `ErlangError`.\n\nSynonyms:\nbravo/bag.tab2file\nbravo/bag tab2file","url":"bravo/bag.html#tab2file"},{"doc":"bravo/bag","title":"tab2list","content":"pub fn tab2list(bag: Bag(a)) -&gt; List(a)\n Returns a list containing all of the objects in the `Bag`.\n\nSynonyms:\nbravo/bag.tab2list\nbravo/bag tab2list","url":"bravo/bag.html#tab2list"},{"doc":"bravo/bag","title":"take","content":"pub fn take(bag: Bag(a), key: b) -&gt; List(a)\n Returns and removes all objects with `key` in the `Bag`, if any exist.\n\nSynonyms:\nbravo/bag.take\nbravo/bag take","url":"bravo/bag.html#take"},{"doc":"bravo/bag","title":"bravo/bag","content":" This module provides functions to work with `Bag`s","url":"bravo/bag.html"},{"doc":"bravo/dbag","title":"DBag","content":"pub opaque type DBag(t)\n A duplicate bag bravo. Keys may occur multiple times per table, and verbatim copies of an object can be stored.\n\n\nSynonyms:\nbravo/dbag.DBag\nbravo/dbag DBag","url":"bravo/dbag.html#DBag"},{"doc":"bravo/dbag","title":"delete","content":"pub fn delete(dbag: DBag(a)) -&gt; Bool\n Deletes a `DBag`.\n\n Table lifetime is static, and memory is only freed when the owner process is killed! Don&#39;t forget to call this function!\n\nSynonyms:\nbravo/dbag.delete\nbravo/dbag delete","url":"bravo/dbag.html#delete"},{"doc":"bravo/dbag","title":"delete_all_objects","content":"pub fn delete_all_objects(dbag: DBag(a)) -&gt; Nil\n Deletes all objects in the `DBag`. This is atomic and isolated.\n\nSynonyms:\nbravo/dbag.delete_all_objects\nbravo/dbag delete_all_objects","url":"bravo/dbag.html#delete_all_objects"},{"doc":"bravo/dbag","title":"delete_key","content":"pub fn delete_key(dbag: DBag(a), key: b) -&gt; Nil\n Deletes all objects addressed by `key`, if any exist. If nothing does, this does nothing.\n\nSynonyms:\nbravo/dbag.delete_key\nbravo/dbag delete_key","url":"bravo/dbag.html#delete_key"},{"doc":"bravo/dbag","title":"delete_object","content":"pub fn delete_object(dbag: DBag(a), object: a) -&gt; Nil\n Deletes a specific object in the `DBag`. Other objects with the same key are unaffected.\n\n If there are multiple of the same object, then they will all be deleted.\n\nSynonyms:\nbravo/dbag.delete_object\nbravo/dbag delete_object","url":"bravo/dbag.html#delete_object"},{"doc":"bravo/dbag","title":"file2tab","content":"pub fn file2tab(\n  filename: String,\n  verify: Bool,\n  decoder: fn(Dynamic) -&gt; Result(a, b),\n) -&gt; Result(DBag(a), BravoError)\n Creates a `Ddbag` from file `filename` that was previously created by `tab2file`.\n\n For type safety reasons, a dynamic decoder must be provided, and the decoder must not fail for all objects in the table.\n\n If the flag `verify` is set, then checks are performed to ensure the data is correct. This can be slow if `tab2file` was ran with `md5sum` enabled.\n\n Size-1 tuples are handled uniquely internally and are treated as non-tuples when saved to disk, so you must decode the non-tuple version of the type.\n\n Can have error types `DecodeFailure` and `ErlangError`.\n\nSynonyms:\nbravo/dbag.file2tab\nbravo/dbag file2tab","url":"bravo/dbag.html#file2tab"},{"doc":"bravo/dbag","title":"insert","content":"pub fn insert(dbag: DBag(a), objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `DBag`.\n\n Returns a `Bool` representing if the inserting succeeded.\n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the `keypos` of the `DBag` is greater than the object tuple size or if the input list is empty.\n\n If an object with the same key already exists, then the old object will be overwritten with the new one.\n\nSynonyms:\nbravo/dbag.insert\nbravo/dbag insert","url":"bravo/dbag.html#insert"},{"doc":"bravo/dbag","title":"insert_new","content":"pub fn insert_new(dbag: DBag(a), objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `DBag`. Unlike `insert`, this cannot overwrite objects and will return false if it tries to do so.\n\n Returns a `Bool` representing if the inserting succeeded.\n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the `keypos` of the `DBag` is greater than the object tuple size or if the input list is empty.\n\nSynonyms:\nbravo/dbag.insert_new\nbravo/dbag insert_new","url":"bravo/dbag.html#insert_new"},{"doc":"bravo/dbag","title":"lookup","content":"pub fn lookup(dbag: DBag(a), key: b) -&gt; List(a)\n Gets a list of objects from a `DBag`.\n\n Returns an list containing the objects, if any match.\n\nSynonyms:\nbravo/dbag.lookup\nbravo/dbag lookup","url":"bravo/dbag.html#lookup"},{"doc":"bravo/dbag","title":"member","content":"pub fn member(dbag: DBag(a), key: b) -&gt; Bool\n Returns whether a `DBag` contains an object at `key`.\n\nSynonyms:\nbravo/dbag.member\nbravo/dbag member","url":"bravo/dbag.html#member"},{"doc":"bravo/dbag","title":"new","content":"pub fn new(\n  name: String,\n  keypos: Int,\n  access: Access,\n) -&gt; Result(DBag(a), BravoError)\n Creates a new ETS table configured as a duplicate bag: keys may occur multiple times per table, and verbatim copies of an object can be stored.\n\n `name`: An atom representing the name of the `DBag`. There may only be one ETS table associated with an atom.\n `keypos`: The index (1-indexed) that represents the key position of the object. This function fails if this is less than 1.\n `access`: Determines how visible the table is to other processes.\n - `Public`: Any process can read or write to the `DBag`.\n - `Protected`: Any process can read the `DBag`. Only the owner process can write to it.\n - `Private`: Only the parent process can read or write to the `DBag`.\n\n Returns a result of the created `DBag`, which can be used by other functions in this module.\n Can have error types `ErlangError` and `NonPositiveKeypos`.\n\nSynonyms:\nbravo/dbag.new\nbravo/dbag new","url":"bravo/dbag.html#new"},{"doc":"bravo/dbag","title":"tab2file","content":"pub fn tab2file(\n  dbag: DBag(a),\n  filename: String,\n  object_count: Bool,\n  md5sum: Bool,\n  sync: Bool,\n) -&gt; Result(Nil, BravoError)\n Saves a `Ddbag` as file `filename` that can later be read back into memory using `file2tab`.\n\n There are three configuration flags with this function:\n - `object_count`: Stores the number of objects in the table in the footer. This can detect truncation.\n - `md5sum`: Stores a md5 checksum of the table and its objects. This can detect even single bitflips, but is computationally expensive.\n - `sync`: Blocks the process until the file has been successfully written.\n\n Can have error type `ErlangError`.\n\nSynonyms:\nbravo/dbag.tab2file\nbravo/dbag tab2file","url":"bravo/dbag.html#tab2file"},{"doc":"bravo/dbag","title":"tab2list","content":"pub fn tab2list(dbag: DBag(a)) -&gt; List(a)\n Returns a list containing all of the objects in the `Ddbag`.\n\nSynonyms:\nbravo/dbag.tab2list\nbravo/dbag tab2list","url":"bravo/dbag.html#tab2list"},{"doc":"bravo/dbag","title":"take","content":"pub fn take(dbag: DBag(a), key: b) -&gt; List(a)\n Returns and removes all objects with `key` in the `DBag`, if any exist.\n\nSynonyms:\nbravo/dbag.take\nbravo/dbag take","url":"bravo/dbag.html#take"},{"doc":"bravo/dbag","title":"bravo/dbag","content":" This module provides functions to work with `DBag`s","url":"bravo/dbag.html"},{"doc":"bravo/oset","title":"OSet","content":"pub opaque type OSet(t)\n An ordered set. Keys may only occur once per table, and objects are ordered (this comes at a performance cost).\n\n In order for a lookup match to occur, entries must _coerce into the same value_. Two values may match even if they have different types.\n\n\nSynonyms:\nbravo/oset.OSet\nbravo/oset OSet","url":"bravo/oset.html#OSet"},{"doc":"bravo/oset","title":"delete","content":"pub fn delete(oset: OSet(a)) -&gt; Bool\n Deletes a `OSet`.\n\n Table lifetime is static, and memory is only freed when the owner process is killed! Don&#39;t forget to call this function!\n\nSynonyms:\nbravo/oset.delete\nbravo/oset delete","url":"bravo/oset.html#delete"},{"doc":"bravo/oset","title":"delete_all_objects","content":"pub fn delete_all_objects(oset: OSet(a)) -&gt; Nil\n Deletes all objects in the `OSet`. This is atomic and isolated.\n\nSynonyms:\nbravo/oset.delete_all_objects\nbravo/oset delete_all_objects","url":"bravo/oset.html#delete_all_objects"},{"doc":"bravo/oset","title":"delete_key","content":"pub fn delete_key(oset: OSet(a), key: b) -&gt; Nil\n Deletes the object addressed by `key`, if it exists. If it doesn&#39;t, this does nothing.\n\nSynonyms:\nbravo/oset.delete_key\nbravo/oset delete_key","url":"bravo/oset.html#delete_key"},{"doc":"bravo/oset","title":"delete_object","content":"pub fn delete_object(oset: OSet(a), object: a) -&gt; Nil\n Deletes a specific object in the `OSet`. This is more useful in `Bag`s and `DBag`s.\n\nSynonyms:\nbravo/oset.delete_object\nbravo/oset delete_object","url":"bravo/oset.html#delete_object"},{"doc":"bravo/oset","title":"file2tab","content":"pub fn file2tab(\n  filename: String,\n  verify: Bool,\n  decoder: fn(Dynamic) -&gt; Result(a, b),\n) -&gt; Result(OSet(a), BravoError)\n Creates a `OSet` from file `filename` that was previously created by `tab2file`.\n\n For type safety reasons, a dynamic decoder must be provided, and the decoder must not fail for all objects in the table.\n\n If the flag `verify` is set, then checks are performed to ensure the data is correct. This can be slow if `tab2file` was ran with `md5sum` enabled.\n\n Size-1 tuples are handled uniquely internally and are treated as non-tuples when saved to disk, so you must decode the non-tuple version of the type.\n\n Can have error types `DecodeFailure` and `ErlangError`.\n\nSynonyms:\nbravo/oset.file2tab\nbravo/oset file2tab","url":"bravo/oset.html#file2tab"},{"doc":"bravo/oset","title":"insert","content":"pub fn insert(oset: OSet(a), objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `OSet`.\n\n Returns a `Bool` representing if the inserting succeeded.\n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the `keypos` of the `OSet` is greater than the object tuple size or if the input list is empty.\n\n If an object with the same key already exists, then the old object will be overwritten with the new one.\n\nSynonyms:\nbravo/oset.insert\nbravo/oset insert","url":"bravo/oset.html#insert"},{"doc":"bravo/oset","title":"insert_new","content":"pub fn insert_new(oset: OSet(a), objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `OSet`. Unlike `insert`, this cannot overwrite objects and will return false if it tries to do so.\n\n Returns a `Bool` representing if the inserting succeeded.\n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the `keypos` of the `OSet` is greater than the object tuple size or if the input list is empty.\n\nSynonyms:\nbravo/oset.insert_new\nbravo/oset insert_new","url":"bravo/oset.html#insert_new"},{"doc":"bravo/oset","title":"lookup","content":"pub fn lookup(oset: OSet(a), key: b) -&gt; Option(a)\n Gets an object from a `OSet`.\n\n Returns an `Option` containing the object, if it exists.\n\nSynonyms:\nbravo/oset.lookup\nbravo/oset lookup","url":"bravo/oset.html#lookup"},{"doc":"bravo/oset","title":"member","content":"pub fn member(oset: OSet(a), key: b) -&gt; Bool\n Returns whether a `OSet` contains an object at `key`.\n\nSynonyms:\nbravo/oset.member\nbravo/oset member","url":"bravo/oset.html#member"},{"doc":"bravo/oset","title":"new","content":"pub fn new(\n  name: String,\n  keypos: Int,\n  access: Access,\n) -&gt; Result(OSet(a), BravoError)\n Creates a new ETS table configured as an ordered set: keys may only occur once per table, and objects are ordered (this comes at a performance cost).\n\n `name`: An atom representing the name of the `OSet`. There may only be one ETS table associated with an atom.\n `keypos`: The index (1-indexed) that represents the key position of the object. This function fails if this is less than 1.\n `access`: Determines how visible the table is to other processes.\n - `Public`: Any process can read or write to the `OSet`.\n - `Protected`: Any process can read the `OSet`. Only the owner process can write to it.\n - `Private`: Only the parent process can read or write to the `OSet`.\n\n Returns a result of the created `OSet`, which can be used by other functions in this module.\n Can have error types `ErlangError` and `NonPositiveKeypos`.\n\nSynonyms:\nbravo/oset.new\nbravo/oset new","url":"bravo/oset.html#new"},{"doc":"bravo/oset","title":"tab2file","content":"pub fn tab2file(\n  oset: OSet(a),\n  filename: String,\n  object_count: Bool,\n  md5sum: Bool,\n  sync: Bool,\n) -&gt; Result(Nil, BravoError)\n Saves a `OSet` as file `filename` that can later be read back into memory using `file2tab`.\n\n There are three configuration flags with this function:\n - `object_count`: Stores the number of objects in the table in the footer. This can detect truncation.\n - `md5sum`: Stores a md5 checksum of the table and its objects. This can detect even single bitflips, but is computationally expensive.\n - `sync`: Blocks the process until the file has been successfully written.\n\n Can have error type `ErlangError`.\n\nSynonyms:\nbravo/oset.tab2file\nbravo/oset tab2file","url":"bravo/oset.html#tab2file"},{"doc":"bravo/oset","title":"tab2list","content":"pub fn tab2list(oset: OSet(a)) -&gt; List(a)\n Returns a list containing all of the objects in the `OSet`.\n\n The list returned is ordered.\n\nSynonyms:\nbravo/oset.tab2list\nbravo/oset tab2list","url":"bravo/oset.html#tab2list"},{"doc":"bravo/oset","title":"take","content":"pub fn take(oset: OSet(a), key: b) -&gt; Option(a)\n Returns and removes an object at `key` in the `OSet`, if such object exists.\n\nSynonyms:\nbravo/oset.take\nbravo/oset take","url":"bravo/oset.html#take"},{"doc":"bravo/oset","title":"bravo/oset","content":" This module provides functions to work with `OSet`s","url":"bravo/oset.html"},{"doc":"bravo/uset","title":"USet","content":"pub opaque type USet(t)\n An unordered set. Keys may only occur once per table, and objects are unordered.\n\n In order for a lookup match to occur, entries must have the same value _and type_.\n\n\nSynonyms:\nbravo/uset.USet\nbravo/uset USet","url":"bravo/uset.html#USet"},{"doc":"bravo/uset","title":"delete","content":"pub fn delete(uset: USet(a)) -&gt; Bool\n Deletes a `USet`.\n\n Table lifetime is static, and memory is only freed when the owner process is killed! Don&#39;t forget to call this function!\n\nSynonyms:\nbravo/uset.delete\nbravo/uset delete","url":"bravo/uset.html#delete"},{"doc":"bravo/uset","title":"delete_all_objects","content":"pub fn delete_all_objects(uset: USet(a)) -&gt; Nil\n Deletes all objects in the `USet`. This is atomic and isolated.\n\nSynonyms:\nbravo/uset.delete_all_objects\nbravo/uset delete_all_objects","url":"bravo/uset.html#delete_all_objects"},{"doc":"bravo/uset","title":"delete_key","content":"pub fn delete_key(uset: USet(a), key: b) -&gt; Nil\n Deletes the object addressed by `key`, if it exists. If it doesn&#39;t, this does nothing.\n\nSynonyms:\nbravo/uset.delete_key\nbravo/uset delete_key","url":"bravo/uset.html#delete_key"},{"doc":"bravo/uset","title":"delete_object","content":"pub fn delete_object(uset: USet(a), object: a) -&gt; Nil\n Deletes a specific object in the `USet`. This is more useful in `Bag`s and `DBag`s.\n\nSynonyms:\nbravo/uset.delete_object\nbravo/uset delete_object","url":"bravo/uset.html#delete_object"},{"doc":"bravo/uset","title":"file2tab","content":"pub fn file2tab(\n  filename: String,\n  verify: Bool,\n  decoder: fn(Dynamic) -&gt; Result(a, b),\n) -&gt; Result(USet(a), BravoError)\n Creates a `USet` from file `filename` that was previously created by `tab2file`.\n\n For type safety reasons, a dynamic decoder must be provided, and the decoder must not fail for all objects in the table.\n\n If the flag `verify` is set, then checks are performed to ensure the data is correct. This can be slow if `tab2file` was ran with `md5sum` enabled.\n\n Size-1 tuples are handled uniquely internally and are treated as non-tuples when saved to disk, so you must decode the non-tuple version of the type.\n\n Can have error types `DecodeFailure` and `ErlangError`.\n\nSynonyms:\nbravo/uset.file2tab\nbravo/uset file2tab","url":"bravo/uset.html#file2tab"},{"doc":"bravo/uset","title":"insert","content":"pub fn insert(uset: USet(a), objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `USet`.\n\n Returns a `Bool` representing if the inserting succeeded.\n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the `keypos` of the `USet` is greater than the object tuple size or if the input list is empty.\n\n If an object with the same key already exists, then the old object will be overwritten with the new one.\n\nSynonyms:\nbravo/uset.insert\nbravo/uset insert","url":"bravo/uset.html#insert"},{"doc":"bravo/uset","title":"insert_new","content":"pub fn insert_new(uset: USet(a), objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `USet`. Unlike `insert`, this cannot overwrite objects and will return false if it tries to do so.\n\n Returns a `Bool` representing if the inserting succeeded.\n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the `keypos` of the `USet` is greater than the object tuple size or if the input list is empty.\n\nSynonyms:\nbravo/uset.insert_new\nbravo/uset insert_new","url":"bravo/uset.html#insert_new"},{"doc":"bravo/uset","title":"lookup","content":"pub fn lookup(uset: USet(a), key: b) -&gt; Option(a)\n Gets an object from a `USet`.\n\n Returns an `Option` containing the object, if it exists.\n\nSynonyms:\nbravo/uset.lookup\nbravo/uset lookup","url":"bravo/uset.html#lookup"},{"doc":"bravo/uset","title":"member","content":"pub fn member(uset: USet(a), key: b) -&gt; Bool\n Returns whether a `USet` contains an object at `key`.\n\nSynonyms:\nbravo/uset.member\nbravo/uset member","url":"bravo/uset.html#member"},{"doc":"bravo/uset","title":"new","content":"pub fn new(\n  name: String,\n  keypos: Int,\n  access: Access,\n) -&gt; Result(USet(a), BravoError)\n Creates a new ETS table configured as a set: keys may only occur once per table, and objects are unordered.\n\n `name`: An atom representing the name of the `USet`. There may only be one ETS table associated with an atom.\n `keypos`: The index (1-indexed) that represents the key position of the object. This function fails if this is less than 1.\n `access`: Determines how visible the table is to other processes.\n - `Public`: Any process can read or write to the `USet`.\n - `Protected`: Any process can read the `USet`. Only the owner process can write to it.\n - `Private`: Only the parent process can read or write to the `USet`.\n\n Returns a result of the created `USet`, which can be used by other functions in this module.\n Can have error types `ErlangError` and `NonPositiveKeypos`.\n\nSynonyms:\nbravo/uset.new\nbravo/uset new","url":"bravo/uset.html#new"},{"doc":"bravo/uset","title":"tab2file","content":"pub fn tab2file(\n  uset: USet(a),\n  filename: String,\n  object_count: Bool,\n  md5sum: Bool,\n  sync: Bool,\n) -&gt; Result(Nil, BravoError)\n Saves a `USet` as file `filename` that can later be read back into memory using `file2tab`.\n\n There are three configuration flags with this function:\n - `object_count`: Stores the number of objects in the table in the footer. This can detect truncation.\n - `md5sum`: Stores a md5 checksum of the table and its objects. This can detect even single bitflips, but is computationally expensive.\n - `sync`: Blocks the process until the file has been successfully written.\n\n Can have error type `ErlangError`.\n\nSynonyms:\nbravo/uset.tab2file\nbravo/uset tab2file","url":"bravo/uset.html#tab2file"},{"doc":"bravo/uset","title":"tab2list","content":"pub fn tab2list(uset: USet(a)) -&gt; List(a)\n Returns a list containing all of the objects in the `USet`.\n\nSynonyms:\nbravo/uset.tab2list\nbravo/uset tab2list","url":"bravo/uset.html#tab2list"},{"doc":"bravo/uset","title":"take","content":"pub fn take(uset: USet(a), key: b) -&gt; Option(a)\n Returns and removes an object at `key` in the `USet`, if such object exists.\n\nSynonyms:\nbravo/uset.take\nbravo/uset take","url":"bravo/uset.html#take"},{"doc":"bravo/uset","title":"bravo/uset","content":" This module provides functions to work with `USet`s","url":"bravo/uset.html"}]);